//
// Created by haridev on 10/5/23.
//

#ifndef DLIO_PROFILER_CONSTANTS_H
#define DLIO_PROFILER_CONSTANTS_H
#define DLIO_PROFILER_CONFIGURATION "DLIO_PROFILER_CONFIGURATION"
#define DLIO_PROFILER_ENABLE "DLIO_PROFILER_ENABLE"
#define DLIO_PROFILER_GOTCHA_PRIORITY "DLIO_PROFILER_GOTCHA_PRIORITY"
#define DLIO_PROFILER_LOG_LEVEL "DLIO_PROFILER_LOG_LEVEL"
#define DLIO_PROFILER_LOG_FILE "DLIO_PROFILER_LOG_FILE"
#define DLIO_PROFILER_INIT "DLIO_PROFILER_INIT"
#define DLIO_PROFILER_INIT_COUNT "DLIO_PROFILER_INIT_COUNT"
#define DLIO_PROFILER_DATA_DIR "DLIO_PROFILER_DATA_DIR"
#define DLIO_PROFILER_DATA_DIR_DELIMITER ':'
#define DLIO_PROFILER_SET_CORE_AFFINITY "DLIO_PROFILER_SET_CORE_AFFINITY"
#define DLIO_PROFILER_ERROR "DLIO_PROFILER_ERROR"
#define DLIO_PROFILER_INC_METADATA "DLIO_PROFILER_INC_METADATA"
#define DLIO_PROFILER_DISABLE_POSIX "DLIO_PROFILER_DISABLE_POSIX"
#define DLIO_PROFILER_DISABLE_STDIO "DLIO_PROFILER_DISABLE_STDIO"
#define DLIO_PROFILER_DISABLE_IO "DLIO_PROFILER_DISABLE_IO"
#define DLIO_PROFILER_TRACE_COMPRESSION "DLIO_PROFILER_TRACE_COMPRESSION"
#define DLIO_PROFILER_DISABLE_TIDS "DLIO_PROFILER_DISABLE_TIDS"
#define DLIO_PROFILER_BIND_SIGNALS "DLIO_PROFILER_BIND_SIGNALS"
#define DLIO_PROFILER_WRITE_BUFFER_SIZE "DLIO_PROFILER_WRITE_BUFFER_SIZE"
#define DLIO_PROFILER_ALL_FILES "all"

static const int EVENT_TYPE_SIZE = 128;
static const unsigned int DLP_PATH_MAX = 1024 * 4;
static const char SEPARATOR = ';';

#endif //DLIO_PROFILER_CONSTANTS_H
