//
// Created by haridev on 10/7/23.
//

#ifndef DLIO_PROFILER_TYPEDEF_H
#define DLIO_PROFILER_TYPEDEF_H
typedef unsigned long long int TimeResolution;
typedef unsigned long int ThreadID;
typedef unsigned long int ProcessID;
typedef char* EventType;
typedef const char* ConstEventType;
#endif //DLIO_PROFILER_TYPEDEF_H
